class Box():

    def __init__(self, x=0, y=0, w=1, h=1):
        """Accept arguments to define our box, and store them."""
        self.x = x
        self.y = y
        self.w = w
        self.h = h

    def __repr__(self):
        return "Box(%s, %s, %s, %s)" % (self.x, self.y, self.w, self.h)


many_boxes = [Box() for i in range(5000)]

pages = [Box(0, i * 55, 30, 50) for i in range(10)]

# We add a "separation" constant so you can see the boxes individually
separation = .2


def layout(_boxes):
    # Because we modify the box list, we will work on a copy
    boxes = _boxes[:]
    # We start at page 0
    page = 0
    # The 1st box should be placed in the correct page
    previous = boxes.pop(0)
    previous.x = pages[page].x
    previous.y = pages[page].y
    while boxes:
        # We take the new 1st box
        box = boxes.pop(0)
        # And put it next to the other
        box.x = previous.x + previous.w + separation
        # At the same vertical location
        box.y = previous.y
        # But if it's too far to the right...
        if (box.x + box.w) > (pages[page].x + pages[page].w):
            # We go all the way left and a little down
            box.x = pages[page].x
            box.y = previous.y + previous.h + separation

        # But if we go too far down
        if box.y + box.h > pages[page].y + pages[page].h:
            # We go to the next page
            page += 1
            # And put the box at the top-left
            box.x = pages[page].x
            box.y = pages[page].y

        previous = box


layout(many_boxes)

import svgwrite


def draw_boxes(boxes, fname, size):
    dwg = svgwrite.Drawing(fname, profile="full", size=size)
    # Draw the pages
    for page in pages:
        dwg.add(
            dwg.rect(
                insert=(f"{page.x}cm", f"{page.y}cm"),
                size=(f"{page.w}cm", f"{page.h}cm"),
                fill="lightblue",
            )
        )
    # Draw all the boxes
    for box in boxes:
        dwg.add(
            dwg.rect(
                insert=(f"{box.x}cm", f"{box.y}cm"),
                size=(f"{box.w}cm", f"{box.h}cm"),
                fill="red",
            )
        )
    dwg.save()


draw_boxes(many_boxes, "lesson4.svg", ("100cm", "100cm"))

pages = [Box(i * 35, 0, 30, 50) for i in range(10)]
layout(many_boxes)
draw_boxes(many_boxes, "lesson4_side_by_side.svg", ("100cm", "50cm"))


from random import randint

pages = [
    Box(i * 35, 0, 30 - randint(0, 10), 50 - randint(0, 30))
    for i in range(10)
]
layout(many_boxes)
draw_boxes(many_boxes, "lesson4_random_sizes.svg", ("100cm", "50cm"))

many_boxes = [Box(w=1 + randint(-5, 5) / 10) for i in range(5000)]
layout(many_boxes)
draw_boxes(
    many_boxes, "lesson4_random_box_sizes.svg", ("100cm", "50cm")
)
